package formats;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

/**
 * Diese Klasse enthlt Hilfsmethoden zum Formatieren von Zahlen, Arrays und Strings
 * zu Strings fr Ausgaben.<br/>
 * Diese Klasse kann nicht instanziert werden.
 * <p/>
 * Created on 11. Juli 2003, 19:03
 * @version 19.06.2004
 * @author  <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 */
public class Formats {
    
    private Formats() {/** empty */}
    
    private static final DecimalFormatSymbols smDecimalFormatSymbols = new DecimalFormatSymbols();
	private static DecimalFormat smDecimalFormat = new DecimalFormat();
	private static final String smLineFeed = "\n";
	private static final String smSpace = " ";
	private static final String smNull = null+"";
	
	/** Konstante: Anzahl Bytes in einem Kilobyte. */
	public static final long smKiloByte = 1024;
	/** Konstante: Anzahl Bytes in einem Megabyte. */
	public static final long smMegaByte = 1024*smKiloByte;
	
    static{
		smDecimalFormatSymbols.setDecimalSeparator('.');
		smDecimalFormat.setDecimalFormatSymbols(smDecimalFormatSymbols);
    }
    
    
    /**
     * Formatiert ein Array von int-Werten zu einem String.
     * Dabei werden die einzelnen Werte durch ein Leerzeichen getrennt
     * und ohne Zeilenumbruch zusammengefgt. Der Wert an Index 0 steht
     * im zurck gegebenen String ganz links.
     * 
	 * @param pArray Der int-Array, der zu formatieren ist.
	 * @return Formatierter String.
	 */
	public static String format(int [] pArray){
    	if (pArray != null){
	        StringBuffer sb = new StringBuffer();
	        for (int i=0; i<pArray.length; i++)
	            sb.append(pArray[i]+smSpace);
	        return sb.toString();
    	}
    	return smNull;
    }
    
	/**
	 * Formatiert einen zweidimensionalen Array von int-Werten zu einem String.
     * Dabei werden die einzelnen Werte durch ein Leerzeichen getrennt.
     * Bsp.:
     * <pre>
     * [0][0] [0][1] [0][2] [0][3] [0][4] \n
     * [1][0] [1][1] [1][2] [1][3] [1][4] \n
     * [2][0] [2][1] [2][2] [2][3] [2][4] \n
     * </pre>
     * Die einzelnen Zeilen werden durch einen Zeilenvorschub abgeschlossen
     *  
	 * @param pArray Der zwei dimensionale int-Array, der zu formatieren ist.
	 * @return Formatierter String.
	 */
	public static String format(int [][] pArray){
		if (pArray != null){
			StringBuffer sb = new StringBuffer();
			for (int i=0; i<pArray.length; i++)
				sb.append(format(pArray[i])+smLineFeed);
			return sb.toString();
		}
		return smNull;
	}
    
	/**
	  * Formatiert ein Array von double-Werten zu einem String.
	  * Dabei werden die einzelnen Werte durch ein Leerzeichen getrennt
	  * und ohne Zeilenumbruch zusammengefgt. Der Wert an Index 0 steht
	  * im zurck gegebenen String ganz links. Jeder double Wert wird mit
	  * zwei Nachkommastellen angegeben.
	  * 
	  * @param pArray Der double-Array, der zu formatieren ist.
	  * @return Formatierter String.
	  */
    public static String format(double [] pArray){
    	if (pArray != null){
	        StringBuffer sb = new StringBuffer();
	        for (int i=0; i<pArray.length; i++)
	            sb.append(format(pArray[i])+smSpace);
	        return sb.toString();
    	}
    	return smNull;
    }
    
	/**
	 * Formatiert einen zweidimensionalen Array von double-Werten zu einem String.
	 * Dabei werden die einzelnen Werte durch ein Leerzeichen getrennt.
	 * Bsp.:
	 * <pre>
	 * [0][0] [0][1] [0][2] [0][3] [0][4] \n
	 * [1][0] [1][1] [1][2] [1][3] [1][4] \n
	 * [2][0] [2][1] [2][2] [2][3] [2][4] \n
	 * </pre>
	 * Die einzelnen Zeilen werden durch einen Zeilenvorschub abgeschlossen
	 *  
	 * @param pArray Der zwei dimensionale double-Array, der zu formatieren ist.
	 * @return Formatierter String.
	 */
    public static String format(double [][] pArray){
    	if (pArray != null){
			StringBuffer sb = new StringBuffer();
			for (int i=0; i<pArray.length; i++)
				sb.append(format(pArray[i])+smLineFeed);
			return sb.toString();
    	}
    	return smNull;
    }
    
    /**
     * Formatiert einen double Wert. Es werden standardmssig zwei Nachkommastellen
     * ausgegeben.
     *   
	 * @param d Der double-Wert, der in einen String umzuwandeln ist.
	 * @return Formatierter String.
	 */
    public static String format(double d){
		return format(d, 2);
    }
    
    /**
     * Formatiert einen double Wert. Es werden die angebene Anzahl an Nachkommastellen benutzt.
     * 
	 * @param d Der double-Wert, der in einen String umzuwandeln ist.
	 * @param pNachkommastellen Anzahl der Nachkommastellen, die bei der Formatierung benutzt werden soll.
	 * @return Formatierter String.
	 */
	public static String format(double d, int pNachkommastellen){
    	StringBuffer lSb = new StringBuffer();
    	lSb.append("0");
    	if (pNachkommastellen>0){
			lSb.append(".");
			for (int i=0; i<pNachkommastellen; i++)
				lSb.append("0");
    	}
		
		smDecimalFormat.applyPattern("0.00");
		lSb = new StringBuffer();
		lSb.append(smDecimalFormat.format(d));

		// berflssige nullen am ende lschen
		while (lSb.charAt(lSb.length()-1)=='0') lSb.deleteCharAt(lSb.length()-1);
		// wenn letztes zeichen ein . ist, war die zahl ganzzahlig
		if (lSb.charAt(lSb.length()-1)=='.') lSb.deleteCharAt(lSb.length()-1);
		
		return lSb.toString();
    }
    
    /**
     * Formatiert die Lnge eines Strings.
     * Ist der bergebene String krzer als der Parameter minlen, so wird
     * der String mit Leerzeichen aufgefllt. Ist der bergebene String
     * lnger als der Parameter maxlen, so wird der String bei 
     * maxlen abgeschitten.
     * 
	 * @param text Der zu formatierende String.
	 * @param minlen Die Mindestlnge fr den String, der zurckgegeben wird.
	 * @param maxlen Die maximale Lnge fr den String, der zurckgegeben wird.
	 * @return Formatierter String.
	 */
	public static String format(String text, int minlen, int maxlen){
        StringBuffer lFormatted = new StringBuffer();
        if (text.length()>maxlen) lFormatted.append(text.substring(0, maxlen));
        else lFormatted.append(text);
        while (lFormatted.length()<minlen) lFormatted.append(smSpace);
        
        return lFormatted.toString();
    }
	
	/**
     * Formatiert die Lnge eines Strings.
     * Ist der bergebene String krzer als der Parameter len, so wird
     * der String mit Leerzeichen aufgefllt. Ist der bergebene String
     * lnger als der Parameter len, so wird der String bei 
     * len abgeschitten.
     * 
	 * @param text Zu formatierender String.
	 * @param len Lnge des zurckzugebenden Strings.
	 * @return Formatierter String mit der Lnge len.
	 */
	public static String format(String text, int len){
		return format(text, len, len);
	}
	
	/**
	 * Formatiert ein Array von Strings zu einem String.
	 * Dabei werden die einzelnen Strings durch Zeilenumbrche getrennt.
	 * 
	 * @param pText Array von Strings, der zu formatieren ist. 
	 * @return Formatierter String.
	 */
	public static String format(String[] pText){
		if (pText != null){
			StringBuffer lFormatted = new StringBuffer();
			for (int i=0; i<pText.length; i++)
				lFormatted.append(pText[i]+"\n");
			return lFormatted.toString();
		}
		return smNull;
	}
	
	/**
	 * Formatiert die bergebene Zahl als eine Byte-Angabe.
	 * Ist der Parameter pBytes grsser als ein Megabyte,
	 * wird ein String der Form "x,xx MB" zurckgeliefert.
	 * Ist er nur grsser als ein Kilobyte, wird "x,xx kB"
	 * zurckgeliefert.
	 * Ist er kleiner als ein Kilobyte, wird "xxx bytes" zurckgeliefert.
	 * 
	 * @param pBytes Byteangabe.
	 * @return Formatierter String.
	 */
	public static String formatBytes(long pBytes){
		if (pBytes > smMegaByte)
			return format(((double)pBytes) / ((double)smMegaByte))+" MB";
		else if (pBytes > smKiloByte)
			return format(((double)pBytes) / ((double)smKiloByte))+" kB";
		else return pBytes+" bytes";
	}
	
	public static void main(String argv[]){
		double [] a_double = {
			-1.02348532244, 
			Math.sqrt(2.0), 
			1.2, 
			-3.1, 
			(1D/3D), 
			-0.1, 
			0.1, 
			.000000000001,
			-0.000000000001,
			2.000000000001, 
			-2.000000000001,
			0.0000000};
		int [] a_int = {-1, 3, 0 , 1, 3, 4, 3242, 234234234};
		System.out.println(format(a_double));
		System.out.println(format(a_int));
	}
}
