package mscJNeuralNet.connectors;

import java.util.ResourceBundle;

import observerPattern.Observable;

import formats.Formats;

import logger.Log;
import mscJNeuralNet.net.Net;

/**
 * Kantengewichtsinitialisierung mit Zufallswerten.
 * Jedem Kantengewicht wird ein gleichverteilter Zufallswert 
 * aus dem Bereich [-x; +x] zugewiesen. Der Parameter x 
 * (maximale/minimale Gewichtsgre) wird beim Aufruf als erster Parameter 
 * der Methode {@link #connectNet(Net, double[])}mit bergeben.
 * <p/>
 * Im folgenden Beispiel wird der RandomNetConnector benutzt, um die Kantengewichte
 * mit Zufallswerten im Bereich [-2; +2] zu initialisieren:<br/>
 * <pre>
 * int [] lLayerSizes = {2, 1}; 
 * Net myNet = new Net();
 * INetConnector lRndConnector = new RandomNetConnector();
 * int [] lParameter = {2D};
 * lRndConnector.connectNet(myNet, lParameter); 
 * <p/>
 * Created on 02.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 02.06.2004
 * @see INetConnector
 */
public class RandomNetConnector implements INetConnector{
	private static final ResourceBundle smResourceBundle = ResourceBundle.getBundle("debug");
	private static final boolean smLogMethodCall;
	static{
		if (smResourceBundle.getString("showMethodCall.NetConnector").equalsIgnoreCase("true"))
			smLogMethodCall = true;
		else smLogMethodCall = false;
	}
	
	/**
	 * Der Wert in der Arrayposition 0 des bergebenen Parameterarrays wird als
	 * maximale / minimale Gewichtsgre gewertet. Der Wert sollte positiv sein,
	 * negative Werte werden per Zufall erzeugt.
	 * Nach der Initialisierung werden die Observer des Netzes benachrichtigt.
	 * 
	 * @see mscJNeuralNet.connectors.INetConnector#connectNet(Net, double[])
	 * @see observerPattern.ObserverManager#notifyObservers(Observable)
	 */
	public void connectNet(Net pNet, double [] pParameters) {
		double[][] lWeights = pNet.getWeights();
		int [][] lLayerNeurons = pNet.getLayerNeurons();
		double lWeightScaleFactor = pParameters[0];
		
		if (smLogMethodCall) Log.getInstance().writeln(this.getClass().getName()+".createConnections("+Formats.format(pParameters)+")");
		
		// weightmatrix entsprechend mLayerNeurons erzeugen
		
		int lInputtingLayer, lOutputtingLayer;
		int lInputtingNeuronIndex, lOutputtingNeuronIndex;
		
		// first connect BIAS to all but inputlayer
		for (lInputtingLayer = 1; lInputtingLayer<lLayerNeurons.length; lInputtingLayer++){
			for (lInputtingNeuronIndex=0; lInputtingNeuronIndex<lLayerNeurons[lInputtingLayer].length; lInputtingNeuronIndex++){
				lWeights
					[0]
					[lLayerNeurons[lInputtingLayer][lInputtingNeuronIndex]] = Net.getRandomWeightValue(lWeightScaleFactor);
			}
		}

		for (lOutputtingLayer=0; lOutputtingLayer<lLayerNeurons.length-1; lOutputtingLayer++){
			lInputtingLayer = lOutputtingLayer+1;
			for (lOutputtingNeuronIndex=0; lOutputtingNeuronIndex<lLayerNeurons[lOutputtingLayer].length; lOutputtingNeuronIndex++){
				for (lInputtingNeuronIndex=0; lInputtingNeuronIndex<lLayerNeurons[lInputtingLayer].length; lInputtingNeuronIndex++){
					lWeights
						[lLayerNeurons[lOutputtingLayer][lOutputtingNeuronIndex]]
						[lLayerNeurons[lInputtingLayer][lInputtingNeuronIndex]] = Net.getRandomWeightValue(lWeightScaleFactor);
				}
			}
		}
		if (smResourceBundle.getString("showCreateConnections").equalsIgnoreCase("true"))
			Log.getInstance().writeln(Formats.format(lWeights));
		
		pNet.getObserverManager().notifyObservers(pNet);
	}

	public String toString(){	return "Random Connector";}
}
