package mscJNeuralNet.examples;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;

import logger.Log;
import mscJNeuralNet.gui.JNetTrainingAlgorithmControl;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator;
import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomNetConnector;
import mscJNeuralNet.patterns.Patterns;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.RProp;


/**
 * Beispiel zur Benutzung der Klassen 
 * {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics},
 * {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator} und
 * {@link mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter}.
 * <p/>
 * Created on 04.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 04.06.2004
 */
public class TestNetStatistics {

	public static void main(String[] args) {
		try{
			Patterns lPat = Patterns.load("alle+tonleiter-smp.txt");
			Log.getInstance().writeln(lPat.toString());
			int [] layerSizes = {15, 8, 8, 3};
			Net lNet = new Net(layerSizes);
			INetTrainingAlgorithm lAlgoRProp = new RProp();
			INetConnector lRandNetConnector = new RandomNetConnector();
			lAlgoRProp.setNet(lNet);
			double [] lConnectorParameters = {.3}; 
			lRandNetConnector.connectNet(lNet, lConnectorParameters);
			NetPerformanceStatistics lNetStat = NetPerformanceStatisticsCalculator.calculateErrors(lNet, lPat.getInputPatterns(), lPat.getOutputPatterns());
			JFrame fMain = new JFrame();
			fMain.addWindowListener(
					new WindowAdapter() {
						public void windowClosing(WindowEvent e){
							System.exit(0);
						}
					}
			);
			Container content = fMain.getContentPane();
			content.setLayout(new FlowLayout());
			JNetTrainingAlgorithmControl lControl = new JNetTrainingAlgorithmControl();
			content.add(lControl);
			fMain.pack();
			fMain.setVisible(true);
			
			double lSse = 1D;
			while (lAlgoRProp.getCycle() < 1000 && lSse > 0.01){
				Net.train(lPat.getInputPatterns(), lPat.getOutputPatterns(), lAlgoRProp);
				lNetStat = NetPerformanceStatisticsCalculator.calculateErrors(lNet, lPat.getInputPatterns(), lPat.getOutputPatterns());
				lSse = lNetStat.averageSumOfSquaredErrors;
			}
			Log.getInstance().writeln(NetPerformanceReporter.getNetPerformance(lNetStat, lAlgoRProp.getCycle()));
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
}
