package mscJNeuralNet.examples;

import observerPattern.Observable;
import observerPattern.Observer;
import mscJNeuralNet.connectors.INetConnector;
import mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector;
import mscJNeuralNet.net.Net;
import mscJNeuralNet.net.PatternDoesNotMatchNetException;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceReporter;
import mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics;
import mscJNeuralNet.patterns.Patterns;
import mscJNeuralNet.trainer.NetTrainer;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.RProp;


/**
 * Beispiel fr nebenlufiges Trainieren eines Netzes mit der dreidimensionalen Xor-Funktion.
 * <p/>
 * Created on 05.08.2004.
 * @author <a href="http://www.graviton.de">M. Serhat Cinar</a>
 * @version 05.08.2004
 * @see mscJNeuralNet.trainer.NetTrainer
 * @see mscJNeuralNet.net.Net
 * @see mscJNeuralNet.connectors.INetConnector
 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm
 * @see mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatistics
 * @see mscJNeuralNet.netPerformanceStatistics.NetPerformanceStatisticsCalculator
 * @see observerPattern.Observer
 */
public class TestTrippleXorLearnConcurrent implements Observer{
	
	public TestTrippleXorLearnConcurrent() throws PatternDoesNotMatchNetException{
		// 1. Erzeugen der bentigten Klassen
		
		// MLP {@link mscJNeuralNet.net.Net}
		// MLP mit der gewnschten Schichtstruktur erstellen:
		// Eingabeschicht: 3 Neuronen
		// 1. Verdeckte Schicht: 3 Neuronen
		// Ausgabeschicht: 1 Neuron
		int [] layerSizesTrippleXOr = {3, 3, 1};
		//* {@link mscJNeuralNet.net.Net} myNet = new Net(layerSizesTrippleXOr);
		Net myNet = new Net(layerSizesTrippleXOr);

		// BIAS wurde automatisch bercksichtigt.

		// {@link mscJNeuralNet.connectors.INetConnector}
		// Diese Klasse wird zum Initialisieren der Kantengewichte bentigt
		INetConnector lNetConnectionAlgo = new RandomSymmetryBreakingNetConnector();
		
		// {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm}
		// Diese Klasse wird zum Trainieren eines MLP bentigt
		INetTrainingAlgorithm lNetTrainAlgo = new RProp();
		
		// {@link mscJNeuralNet.trainer.NetTrainer}
		// Diese Klasse wird das Netz als nebenlufigen Prozess trainieren.
		NetTrainer lTrainer = new NetTrainer();
		
		
		// 2. Initialisieren des Netzes
		
		// MLP + {@link mscJNeuralNet.connectors.INetConnector}
		// Mit der Instanz von {@link mscJNeuralNet.connectors.INetConnector} inititalisieren.
		// Die Klasse {@link mscJNeuralNet.connectors.RandomSymmetryBreakingNetConnector} bentigt
		// keine eigenen Kontrollparameter und wird daher mit dem Wert null aufgerufen.
		lNetConnectionAlgo.connectNet(myNet, null);
		
		// Nun ist das Netz verbunden und initialisiert.
		
		// {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm}
		// Lernverfahren {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm} mit Netz verbinden.
		lNetTrainAlgo.setNet(myNet);
		
		
		// 3. Trainer konfigurieren
		
		// Dem Trainer das Lernverfahren und das MLP mitteilen 
		lTrainer.setTrainingAlgorithm(lNetTrainAlgo);
		// 100 Lernschritte lang trainieren
		lTrainer.setTargetCycles(100);
		// oder bis Fehlerwert unter 0.01
		lTrainer.setTargetError(0.01);
		// Fehlerwert soll vom Typ SSE sein
		lTrainer.setTargetErrorType(NetPerformanceStatistics.ERRORTYPE_averageSumOfSquaredError);
		// Lerndaten immer in derselben Reihenfolge prsentieren
		lTrainer.setUseRandomizedPatternOrder(false);
		
		// Dieses Programm als Observer des Trainers anmelden
		lTrainer.getObserverManager().addObserver(this);
		
		
		// 4. Trainieren des Netzes
		
		// TRAININGSDATEN
		// Trainingsdaten bereitstellen:
		// Trainingsdaten in ein Patterns Objekt bertragen
		// Komforbaler ist es, die Lerndaten in einer Textdatei 
		// zu speichern und hier zu laden
		Patterns lTrippleXorPat = new Patterns(3, 1, 8);
		// {-1,-1,-1}
		lTrippleXorPat.setInputToken(0, 0, -1);
		lTrippleXorPat.setInputToken(1, 0, -1);
		lTrippleXorPat.setInputToken(2, 0, -1);
		// {-1}
		lTrippleXorPat.setOutputToken(0, 0, -1);
		
		// {-1,-1,1}
		lTrippleXorPat.setInputToken(0, 1, -1);
		lTrippleXorPat.setInputToken(1, 1, -1);
		lTrippleXorPat.setInputToken(2, 1, 1);
		// {1}
		lTrippleXorPat.setOutputToken(0, 1, 1);
		
		// {-1,1,-1}
		lTrippleXorPat.setInputToken(0, 2, -1);
		lTrippleXorPat.setInputToken(1, 2, 1);
		lTrippleXorPat.setInputToken(2, 2, -1);
		// {1}
		lTrippleXorPat.setOutputToken(0, 2, 1);
		
		// {-1,1,1}
		lTrippleXorPat.setInputToken(0, 3, -1);
		lTrippleXorPat.setInputToken(1, 3, 1);
		lTrippleXorPat.setInputToken(2, 3, 1);
		// {-1}
		lTrippleXorPat.setOutputToken(0, 3, -1);

		// {1,-1,-1}
		lTrippleXorPat.setInputToken(0, 4, 1);
		lTrippleXorPat.setInputToken(1, 4, -1);
		lTrippleXorPat.setInputToken(2, 4, -1);
		// {1}
		lTrippleXorPat.setOutputToken(0, 4, 1);
		
		// {1,-1,1}
		lTrippleXorPat.setInputToken(0, 5, 1);
		lTrippleXorPat.setInputToken(1, 5, -1);
		lTrippleXorPat.setInputToken(2, 5, 1);
		// {-1}
		lTrippleXorPat.setOutputToken(0, 5, -1);
		
		// {1,1,-1}
		lTrippleXorPat.setInputToken(0, 6, 1);
		lTrippleXorPat.setInputToken(1, 6, 1);
		lTrippleXorPat.setInputToken(2, 6, -1);
		// {-1}
		lTrippleXorPat.setOutputToken(0, 6, -1);

		// {1,1,1}
		lTrippleXorPat.setInputToken(0, 7, 1);
		lTrippleXorPat.setInputToken(1, 7, 1);
		lTrippleXorPat.setInputToken(2, 7, 1);
		// {1}
		lTrippleXorPat.setOutputToken(0, 7, 1);
		
		// Trainingsdatenmenge dem Trainer mitteilen
		lTrainer.setTrainingPatterns(lTrippleXorPat);
		
		// Training beginnen
		lTrainer.start();
	}
	
	public static void main(String [] args) {
		try{
			new TestTrippleXorLearnConcurrent();
		}
		catch (PatternDoesNotMatchNetException e){
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		// Diese Methode wird vom NetTrainer nach jedem Lernschritt aufgerufen.
		// Das bergebene Observable Objekt ist der NetTrainer selbst
		if (pObservable instanceof NetTrainer){
			NetTrainer lTrainer = (NetTrainer) pObservable;
			// Prfen, ob das Training vorbei ist
			if (lTrainer.hasFinished()){
				System.out.println("Training fertig.");
				System.out.println(
						NetPerformanceReporter.getNetPerformance(
								lTrainer.getLastCalcualtedNetStatistics(), 
								lTrainer.getTrainingAlgorithm().getCycle()
						)
				);
				System.exit(0);
			}
			else{
				// Sonst alle 10 Lernschritte Status ausgeben
				if (lTrainer.getTrainingAlgorithm().getCycle() % 10 == 0){
					System.out.println("Lernschritt: "+lTrainer.getTrainingAlgorithm().getCycle());
					System.out.println("Aktueller Netzfehler: "+
							lTrainer.getTargetErrorTypeString()+" "+
							lTrainer.getLastCalculatedError());
				}
			}
		}
	}
}
