package mscJNeuralNet.gui;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import formats.Formats;

import mscJNeuralNet.gui.subComponents.JLabeledTextField;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.enumerations.TrainingAlgorithmsEnumeration;
import observerPattern.Observable;
import observerPattern.Observer;
import observerPattern.ObserverManager;

/**
 * Grafische Komponente zur Auswahl eines TrainingAlgorithmus.<p/>
 * <img src="../../images/jnettrainingalgorithmcontrol.gif"><p/>
 * 
 * Created on 04.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 04.06.2004
 */
public class JNetTrainingAlgorithmControl extends JPanel implements Observer, Observable{
	private static final ResourceBundle smMSCResourceBundle = ResourceBundle.getBundle("mscJNeuronalNet_Text");
	
	private final ObserverManager mObserverManager = new ObserverManager(); 
	
	private INetTrainingAlgorithm mTrainingAlgorithm = null;
	private JPanel mPMain;
	private JLabeledTextField mLTfParameters[] = null;
	private String [] mParameterNames = null;
	private double [] mParameterValues = null;
	private JButton mBResetParameters;
	private JComboBox mCBAlgortihms; 
	private final Vector mAlgorithms = new Vector();
	private final Vector mAlgorithmNames = new Vector();

	private static final String smColon = ":";
	
	public JNetTrainingAlgorithmControl(){
		super();
		setBorder(
			BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.BLACK),
				smMSCResourceBundle.getString("TRAINING_ALGORITHM")
			)
		);
		
		mPMain = new JPanel();
		
		Enumeration mAlgosEnum = new TrainingAlgorithmsEnumeration();
		INetTrainingAlgorithm lAlgo;
		while (mAlgosEnum.hasMoreElements()){
			lAlgo = (INetTrainingAlgorithm) mAlgosEnum.nextElement();
			mAlgorithms.add(lAlgo);
			mAlgorithmNames.add(lAlgo.getAlgorithmName());
		}
		
		mBResetParameters = new JButton(smMSCResourceBundle.getString("RESET_PARAMETERS"));
		mBResetParameters.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent arg0) {
					mTrainingAlgorithm.setLearningParameters(
						mTrainingAlgorithm.getDefaultLearningParameters());
				}
			} 
		);
		
		mCBAlgortihms = new JComboBox(mAlgorithmNames);
		mCBAlgortihms.addActionListener(
			new ActionListener(){
				public void actionPerformed(ActionEvent e) {
					int lSelectedAlgo = mCBAlgortihms.getSelectedIndex();
					if (! ((String) mAlgorithmNames.get(lSelectedAlgo)).equals(mTrainingAlgorithm.getAlgorithmName()))
						setTrainingAlgorithm((INetTrainingAlgorithm) mAlgorithms.get(lSelectedAlgo));
				}
			}
		);
		
		add(mPMain);
		
		setTrainingAlgorithm((INetTrainingAlgorithm) mAlgorithms.get(0));
	}
	
	public void setTrainingAlgorithm(INetTrainingAlgorithm pAlgo){
//		Net lNet = null;
		if (mTrainingAlgorithm != null){
			mTrainingAlgorithm.getObserverManager().removeObserver(this);
//			lNet = mTrainingAlgorithm.getNet();
		}
		
		mTrainingAlgorithm = pAlgo;
		mTrainingAlgorithm.getObserverManager().addObserver(this);
		
		// alte komponenten abmelden
		if (mLTfParameters != null)
			for (int i=0; i<mLTfParameters.length; i++)
				mLTfParameters[i].getObserverManager().removeObserver(this);
		// alte komponenten lschen
		mPMain.removeAll();
		
		mPMain.setLayout(new GridLayout(mTrainingAlgorithm.getLearningParameters().length+2, 1));
		
		// auswahl der combobox aktualisieren, falls algorithmus von aussen kommt.
		for (int i=0; i<mCBAlgortihms.getItemCount(); i++){
			if ( ((String)mCBAlgortihms.getItemAt(i)).equals(mTrainingAlgorithm.getAlgorithmName()))
				mCBAlgortihms.setSelectedIndex(i);
		}
		mPMain.add(mCBAlgortihms);
		
		// parameter felder erzeugen
		mParameterNames = mTrainingAlgorithm.getLearningParameterNames();
		mParameterValues = mTrainingAlgorithm.getLearningParameters();
		mLTfParameters = new JLabeledTextField[mParameterNames.length];
		for (int i=0; i<mParameterNames.length; i++){
			mLTfParameters[i] = new JLabeledTextField(5);
			mLTfParameters[i].getLabel().setText(mParameterNames[i]+smColon);
			mLTfParameters[i].getTextField().setText(Formats.format(mParameterValues[i], 6));
			mLTfParameters[i].getObserverManager().addObserver(this);
			mPMain.add(mLTfParameters[i]);
		}
		
		mPMain.add(mBResetParameters);
		
//		this.updateUI();
				
		mObserverManager.notifyObservers(this);
	}
	
	public INetTrainingAlgorithm getTrainingAlgorithm(){
		return mTrainingAlgorithm;
	}
	
	/* (non-Javadoc)
	 * @see observerPattern.Observer#notify(observerPattern.Observable)
	 */
	public void notify(Observable pObservable) {
		if (isVisible() && pObservable == mTrainingAlgorithm){
			mParameterValues = mTrainingAlgorithm.getLearningParameters();
			for (int i=0; i<mParameterValues.length; i++){
				mLTfParameters[i].getTextField().setText(Formats.format(mParameterValues[i], 6));
			}
		}
		else{
			// meldung stammt von einem jTextField durch return oder focus loss
			mParameterValues = mTrainingAlgorithm.getLearningParameters();
			double lGuiParameterValue;
			for (int i=0; i<mLTfParameters.length; i++){
				if (mLTfParameters[i]==pObservable){
					// hat sich der wert des feldes gendert?
					lGuiParameterValue = Double.parseDouble(mLTfParameters[i].getTextField().getText()); 
					if (lGuiParameterValue != mParameterValues[i]){
						mParameterValues[i] = lGuiParameterValue;
						mTrainingAlgorithm.setLearningParameters(mParameterValues); 
					}
					break;
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see observerPattern.Observable#getObserverManager()
	 */
	public ObserverManager getObserverManager() {
		return mObserverManager;
	}
}
