package mscJNeuralNet.net;


/**
 * Created on 04.07.2005
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 04.07.2005
 */
public class TransferTanh implements ITransferfunction{

	/**
	 * Berechnet den Tangens Hyperbolicus des gegebenen Wertes.
	 * 
	 * @param x Der Wert, dessen Tangens Hyperbolicus berechnet werden soll.
	 * @return Der Tangens Hyperbolicus des gegebenen Wertes.
	 * @see mscJNeuralNet.net.ITransferfunction#eval(double)
	 */
	public double eval(double x) {
		if (x == 0.0D) return x;
		double a = x;
		if (a < 0.0D) a = Math.abs(x);
		a = Math.exp(2D * a);
		if (x < 0.0D) return -(1.0D - 2D / (a + 1.0D));
		return 1.0D - 2D / (a + 1.0D);
	}

	/**
	 * Berechnet die Ableitung des Tangens Hyperbolicus fr den gegebenen Wert.
	 * Die Ableitung des Tangens Hyperbolicus entspricht (1 - x^2).
	 * 
	 * @param x Der Wert, zu dem die Ableitung des Tangens Hyperbolicus berechnet werden soll.
	 * @return Die Ableitung des Tangens Hyperbolicus fr den gegebenen Wert.
	 * @see mscJNeuralNet.net.ITransferfunction#evalDif(double)
	 */
	public double evalDif(double x) {
		return (1D - x*x);
	}

	/* (non-Javadoc)
	 * @see mscJNeuralNet.net.ITransferfunction#getName()
	 */
	public String getName() {
		return "tanh";
	}

}
