package mscJNeuralNet.trainingAlgorithms;


import java.util.ResourceBundle;

import formats.Formats;


import logger.Log;

/**
 * Offline Backpropagation Lernverfahren mit Momentumterm nach Rumelhart, Hinton und Williams.
 * Implentiert nach der Arbeit
 * <i>D. E. Rumelhart, G. Hinton, R. Williams - 
 * Learning internal representations by error propagation, 1986</i><br/>
 * Backpropagation ist wohl der Klassiker unter den Lernverfahren,
 * hat aber auch die schlechteste Performanz.
 * <p/>
 * Die Aktualisierung der Kantengewichte wird wie folgt berechnet:<br/>
 * <pre>
 *              Backpropagation Term        Momentum Term
 * Delta Wij = -1 * alpha * Gradient Wij + momentum * Delta WijAlt
 * </pre>
 * <p/>
 * Created on 05.04.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 05.06.2004
 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm
 * @see mscJNeuralNet.trainingAlgorithms.AbstractTrainingAlgorithm
 * @see mscJNeuralNet.trainingAlgorithms
 */
public class BackpropagationMomentum extends AbstractTrainingAlgorithm{
	private static final ResourceBundle smMSCResourceBundle = ResourceBundle.getBundle("mscJNeuronalNet_Text");
	private static final ResourceBundle smResourceBundle = ResourceBundle.getBundle("debug");
//	private static final boolean smLogCalculateWeightUpdate;
	private static final boolean smLogMethodCall;
	static{
//		if (smResourceBundle.getString("showLearningCalculateWeightUpdateBPG").equalsIgnoreCase("true"))
//			smLogCalculateWeightUpdate = true;
//		else smLogCalculateWeightUpdate = false;
		if (smResourceBundle.getString("showMethodCall.TrainingAlgorithm").equalsIgnoreCase("true"))
			smLogMethodCall = true;
		else smLogMethodCall = false;
	}
	
	private double [][] mWeightChangeOld;
	private double [][] mWeightChange;
	
	private static final String smParameterNames [] = {
		smMSCResourceBundle.getString("ALPHA_LEARNING_RATE"),
		smMSCResourceBundle.getString("MOMENTUM")
	};
	private static final double [] smParameterDefaultValues = {1.2, .6};
	
	private double mAlpha = smParameterDefaultValues[0];
	private double mMomentum = smParameterDefaultValues[1];
	
	/* (non-Javadoc)
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm#init()
	 */
	public void init(){
		super.init();
		
		if (smLogMethodCall) Log.getInstance().writeln(this.getClass().getName()+".init()");
		// init gradients
		mWeightChange = new double [getNet().getAmountNeurons()][getNet().getAmountNeurons()];
		mWeightChangeOld = new double [getNet().getAmountNeurons()][getNet().getAmountNeurons()];
		for (int x=0; x<getNet().getAmountNeurons(); x++)
			for (int y=0; y<getNet().getAmountNeurons(); y++){
				mWeightChangeOld[x][y] = 0D;
				mWeightChange[x][y] = 0D;
			}
		setCycle(0);
	}
	
	/* (non-Javadoc)
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm#applyWeightUpdate(int, int)
	 */
	public void applyWeightUpdate(int pOutputting, int pInputting){
		super.applyWeightUpdate(pOutputting, pInputting);

		mWeightChange[pOutputting][pInputting] =
			-1*mAlpha*getActualGradients()[pOutputting][pInputting]+
			mMomentum*mWeightChangeOld[pOutputting][pInputting];
		getNet().getWeights()[pOutputting][pInputting] += mWeightChange[pOutputting][pInputting];
		mWeightChangeOld[pOutputting][pInputting] = mWeightChange[pOutputting][pInputting];
	}
	
	/**
	 * Dieses Lernverfahren untersttzt folgende Parameter: <br/>
	 * <pre>
	 * Index  Bezeichnung  Funktion
	 *   0    Alpha        Lernschrittweite
	 *   1    Momentum     Faktor fr den Momentumterm
	 * </pre>
	 * Am Ende der Methode werden die Observer benachrichtigt.
	 *  
	 * @see INetTrainingAlgorithm#setLearningParameters(double[])
	 */
	public void setLearningParameters(double[] pLearningParameters) {
		if (smLogMethodCall) Log.getInstance().writeln(this.getClass().getName()+".setLearningParameters("+Formats.format(pLearningParameters)+")");
		
		if (pLearningParameters.length>0){
			if (pLearningParameters[0] >= 0D) mAlpha = pLearningParameters[0];
			if (pLearningParameters.length>1){
				if (pLearningParameters[1] >= 0D) mMomentum = pLearningParameters[1];
			}
		}
		getObserverManager().notifyObservers(this);
	}

	/**
	 * Dieses Lernverfahren untersttzt folgende Parameter: <br/>
	 * <pre>
	 * Index  Bezeichnung  Funktion
	 *   0    Alpha        Lernschrittweite
	 *   1    Momentum     Faktor fr den Momentumterm
	 * </pre>
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm#getLearningParameters()
	 */
	public double[] getLearningParameters() {
		double [] lParameters = new double[smParameterNames.length];
		lParameters[0] = mAlpha;
		lParameters[1] = mMomentum;
		return lParameters;
	}

	/**
	 * Dieses Lernverfahren untersttzt folgende Parameter: <br/>
	 * <pre>
	 * Index  Bezeichnung  Funktion
	 *   0    Alpha        Lernschrittweite
	 *   1    Momentum     Faktor fr den Momentumterm
	 * </pre>
	 * 
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm#getLearningParameterNames()
	 */
	public String[] getLearningParameterNames() {
		return smParameterNames;
	}
	
	/**
	 * Dieses Lernverfahren untersttzt folgende Parameter: <br/>
	 * <pre>
	 * Index  Bezeichnung  Funktion
	 *   0    Alpha        Lernschrittweite
	 *   1    Momentum     Faktor fr den Momentumterm
	 * </pre>
	 * 
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm#getDefaultLearningParameters()
	 */
	public double [] getDefaultLearningParameters(){
		return smParameterDefaultValues;
	}
	
	/**
	 * Liefert den String "Backprop + Momentum" zurck.
	 * 
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm#getAlgorithmName()
	 */
	public String getAlgorithmName(){
		return "Backprop + Momentum";
	}
	
	public String toString(){
		return "BpgOffline alpha: "+mAlpha+" momentum: "+mMomentum;
	}
}
