package mscJNeuralNet.trainingAlgorithms.enumerations;

import java.util.Enumeration;

import mscJNeuralNet.trainingAlgorithms.BackpropagationMomentum;
import mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm;
import mscJNeuralNet.trainingAlgorithms.QuickProp;
import mscJNeuralNet.trainingAlgorithms.RProp;
import mscJNeuralNet.trainingAlgorithms.RPropWeightDecay;
import mscJNeuralNet.trainingAlgorithms.SuperSABMomentum;

/**
 * Enumeration ber alle verfgbaren Klassen, welche die Schnittstelle
 * {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm} implementieren.<br/>
 * Es werden die {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm} in der folgenden 
 * Reihenfolge zurckgeliefert:
 * <dir>
 * 	<li>{@link mscJNeuralNet.trainingAlgorithms.BackpropagationMomentum}</li>
 *  <li>{@link mscJNeuralNet.trainingAlgorithms.QuickProp}</li>
 *  <li>{@link mscJNeuralNet.trainingAlgorithms.RProp}</li>
 *  <li>{@link mscJNeuralNet.trainingAlgorithms.RPropWeightDecay}</li>
 *  <li>{@link mscJNeuralNet.trainingAlgorithms.SuperSABMomentum}</li>
 * </dir>
 * <p/> 
 * Created on 09.06.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 09.06.2004
 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm
 * @see java.util.Enumeration
 */
public class TrainingAlgorithmsEnumeration implements Enumeration {
	
	private final INetTrainingAlgorithm [] smAlgorithms = {
		new BackpropagationMomentum(),
		new QuickProp(),
		new RProp(),
		new RPropWeightDecay(),
		new SuperSABMomentum()
	};
	private int mIterationPosition = 0;
	
	/* (non-Javadoc)
	 * @see java.util.Enumeration#hasMoreElements()
	 */
	public boolean hasMoreElements() {
		return smAlgorithms.length > mIterationPosition;
	}

	/**
	 * Es wird jeweils ein Objekt des Typs {@link mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm}
	 * zurckgeliefert.
	 * 
	 * @see java.util.Enumeration#nextElement()
	 * @see mscJNeuralNet.trainingAlgorithms.INetTrainingAlgorithm
	 */
	public Object nextElement() {
		return smAlgorithms[mIterationPosition++];
	}

}
