package observerPattern;

/**
 * Die Schnittstelle Observable ist von der zu beobachtenden Klasse zu 
 * implementieren und umfasst lediglich die Methode 
 * public ObserverManager getObserverManager(). 
 * Der ObserverManager ist eine Klasse, die dabei hilft,
 * die Observer eines Observers zu verwalten und diese bei
 * bedarf zu benachrichtigen. Jede Klasse, die beobachtet 
 * werden soll, muss eine Instanz von ObserverManager erzeugen.<br/>
 * Durch folgende einfachen Schritte kann eine Klasse Observable implementieren:<br/>
 * <dir>
 * 	<li>Die Schnittstelle Observable implementieren:<br/>
 * 	  <code>public class myObservable impements Observable...</code>
 *  </li>
 *  <li>Eine Instanz von ObserverManager erzeugen:<br/>
 * 	  <code>private ObserverManager mObserverManager = new ObserverManager();</code>
 *  </li>
 *  <li>In der Methode getObserverManager() diesen ObserverManager zurckgeben:<br/>
 *    <code>
 * 		public ObserverManager getObserverManager(){
 * 			return mObserverManager;
 * 		}</code>
 *  </li>
 *  <li>Bei bedarf die Observer benachrichtigen:<br/>
 *    <code>mObserverManager.notifyObservers(this);</code>
 *  </li>
 * </dir>
 * <p/> 
 * Created on 21.04.2004
 * @author <A HREF="http://www.graviton.de">M. Serhat Cinar</A>
 * @version 21.04.2004
 * @see observerPattern.ObserverManager
 * @see observerPattern.Observer
 */
public interface Observable {
	/**
	 * Liefert den ObserverManager dieses Observables.
	 * 
	 * @return Der ObserverManager dieses Observables.
	 */
	public ObserverManager getObserverManager();
}
